#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)men_cmm.pl 1.28 04/10/08 SMI"  
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------
package MEN_CMM;

use strict;
use variables;

#------------------------------------------------------------------------------
#
# Configure CMM 
#
#------------------------------------------------------------------------------
sub cmm_configure {

  my ($proto_node_id) = @_;

  my $proto_name = COMMON::build_node_name($proto_node_id, "");
  my $proto_name_nic0 = COMMON::build_node_name($proto_node_id, "NIC0");
  my $proto_ip   = COMMON::get_external_ip($proto_node_id);
  
  my $local_cmm_conf = "$WORKING_DIR/cmm_conf";
  
  my $local_nodename = "$WORKING_DIR/nodename_${proto_name}";
  my $remote_nodename = "/etc/nodename";
  
  my $node_name_template;	
      
  # Create a cluster_nodes_table file
  #
  open(CMM_CONFIG, ">$local_cmm_conf") ||
    COMMON::error("Unable to create $local_cmm_conf");
  
  print CMM_CONFIG "#NodeId\tDomain_id\tName\tAttributes\n";
  
  for my $node (0..$#NODE_LIST) {
    
    my $node_id = $NODE_LIST[$node]{id};
    $node_name_template = COMMON::build_node_name($node_id, "");       
    print CMM_CONFIG "${node_id}\t${CLUSTER}\t${node_name_template}\t-\n";
    
  }		
  close(CMM_CONFIG);
  
  # remote copy cluster_node_table
  #
  COMMON::put_file($proto_ip, "$local_cmm_conf", "$NODE_TABLE_FILE");
  
  #
  # create nodename file
  #
  open(NODENAME, ">${local_nodename}") ||
    COMMON::error("Unable to create ${local_nodename}");
  
  # must be nic0 (to allow smosservice to work correctly)
  # set to cgtp nodename at the end of installation
  print NODENAME "${proto_name_nic0}\n";
  
  close(NODENAME);
  
  #
  # remote copy nodename
  #
  COMMON::put_file($proto_ip, "$local_nodename", "$remote_nodename");
}

#--------------------------------------------------------------
#
# installation of CMM packages
#
#--------------------------------------------------------------

sub men_cmm {

  ($NODE_ID) = @_;

  COMMON::print_stage_node("Cluster Membership Monitor (CMM) installation") ;
  COMPONENT::install_men_component($NODE_ID, "CMM", "");

  COMMON::print_action("CMM configuration");
  cmm_configure ($NODE_ID);

}

#--------------------------------------------------------------
#
#  recovery after failure when installing CMM packages
#
#--------------------------------------------------------------

sub recovery_men_cmm {

  ($NODE_ID) = @_;

  COMMON::print_recovery_node("Cluster Membership Monitor (CMM) clean-up") ;
  COMPONENT::remove_men_component($NODE_ID, "CMM") ;
}

#--------------------------------------------------------------
#
#  add new nodes in cluster_nodes_table
#
#--------------------------------------------------------------

sub men_cmm_add_nodes {

  ($NODE_ID) = @_;
  my $node_ip   = COMMON::get_external_ip($NODE_ID);
  my $local_cmm_conf = "$WORKING_DIR/cmm_conf";
  
  COMMON::print_stage_node("Adding new nodes in the cluster nodes table") ;

  # get file on remote node
  COMMON::get_file($node_ip, "$NODE_TABLE_FILE", "$local_cmm_conf");

  # update the file and all new nodes
  open(CMM_CONFIG, ">>$local_cmm_conf") ||
    COMMON::error("Unable to update $local_cmm_conf");
       
  for my $node (0..$#NODE_LIST) {
    if ($NODE_LIST[$node]{concerned}) {
      my $node_id = $NODE_LIST[$node]{id};
      my $node_name_template = COMMON::build_node_name($node_id, "");       
      print CMM_CONFIG "${node_id}\t${CLUSTER}\t${node_name_template}\t-\n";
    }		
  }

  close(CMM_CONFIG);

  # remote copy cluster_node_table
  COMMON::put_file($node_ip, "$local_cmm_conf", "$NODE_TABLE_FILE");
}


#--------------------------------------------------------------
#
#  reload the cluster_nodes_table file on the master
#
#--------------------------------------------------------------

sub men_cmm_reload_file {

  ($NODE_ID) = @_;

  my $node_ip   = COMMON::get_external_ip($NODE_ID);

  COMMON::print_stage_node("Reload the cluster nodes table on the master") ;

  # force the CMM to reload the file (otherwise it will be superseded when
  # the daemon will stop
  COMMON::remote_exec($node_ip, "$CMMADM -c reload") ||
    COMMON::error ( "Unable to reload $NODE_TABLE_FILE" );
  
}

{
}







