#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)men_svm.pl 1.6 04/09/22 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package MEN_SVM;

use strict;
use variables;

#------------------------------------------------------------------------------
#
# creation of metadevice
#
#------------------------------------------------------------------------------

sub configure_svm() {

  my $node_ip = COMMON::get_external_ip($NODE_ID);
  my $node_idx = COMMON::getnode($NODE_ID);
  my $idx;

  # creation of state database replica
  COMMON::slice_search($NODE_ID, "metadb", $TRUE, \$idx);
  my $slice_metadb = $SLICE_LIST[$idx]{block};

  COMMON::print_action("Creating metadevice database on $slice_metadb");
  COMMON::remote_exec($node_ip, "$RMETADB -a -f $slice_metadb") || 
    COMMON::error ( "Unable to create meta database on slice $slice_metadb");

  # creation of each concat metadevice for the current node
  foreach my $i (@{$NODE_LIST[$node_idx]{slice}}) {

    if ($SLICE_LIST[$i]{metadevice} ne "") {
      COMMON::print_action("converting slice $SLICE_LIST[$i]{name} into metadevice $SLICE_LIST[$i]{metadevice}");

      # unmount UFS partition if mounted
      # ignore error, it may stay unmount after a recovery
      if (($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED)
	  &&  ($SLICE_LIST[$i]{mounting_point} ne $SLICE_TAG_REPLICA)) {
	COMMON::print_subaction("umount $SLICE_LIST[$i]{mounting_point}");
	COMMON::remote_exec_ignore($node_ip, "$RUMOUNT $SLICE_LIST[$i]{mounting_point}") ;
      }

      # create the metadevice
      COMMON::print_subaction("creating metadevice $SLICE_LIST[$i]{metadevice}");
      COMMON::remote_exec($node_ip, "$RMETAINIT $SLICE_LIST[$i]{metadevice} 1 1 $SLICE_LIST[$i]{name}") ||
	COMMON::error ( "Unable to create metadevice $SLICE_LIST[$i]{metadevice} on $SLICE_LIST[$i]{name}" );

      #create the filesystem on the raw partition
      if (($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED)
	  &&  ($SLICE_LIST[$i]{mounting_point} ne $SLICE_TAG_REPLICA)) {
	COMMON::print_subaction("creating filesystem on $SLICE_LIST[$i]{raw}");
	COMMON::remote_exec($node_ip, "$RNEWFS $SLICE_LIST[$i]{raw}") ||
	  COMMON::error ( "Unable to create the filesystem on $SLICE_LIST[$i]{raw}");	
      }
    }
  }

}
#------------------------------------------------------------------------------
#
# update /etc/vfstab to replace physical slice name by metadevice name
#
#------------------------------------------------------------------------------

sub update_vfstab_entry {
  my ($vfstab_file, $slice, $metadevice_block, $metadevice_raw ) = @_ ;

  my $current_block;
  my $current_raw;
  my $garbage1;
  my $garbage2;
  my $garbage3;
  my $garbage4;
  my $garbage5;
    
  open (VFSTAB, $vfstab_file ) || COMMON::error("Unable to open $vfstab_file");
  open (VFSTAB_TMP, ">${vfstab_file}.tmp" ) || COMMON::error("Unable to open ${vfstab_file}.tmp");
  
  while ( <VFSTAB> ) {
    
    chop();
    s/#.*//;
    if ($_ eq "") {
      next;
    }

    ($current_block, $current_raw, $garbage1, $garbage2, $garbage3, $garbage4, $garbage5) =
      split (/\t/, $_) ;
    if ( $current_block eq "/dev/dsk/$slice" ) {
      print VFSTAB_TMP "$metadevice_block\t$metadevice_raw\t$garbage1\t$garbage2\t$garbage3\t$garbage4\t$garbage5\n";
    } else {
      print VFSTAB_TMP "$_\n";
    }
    next;       	        
  }
   
  close (VFSTAB);
  close (VFSTAB_TMP);
	
	
  COMMON::exec_cmd ( "$MV ${vfstab_file}.tmp ${vfstab_file}") ||
    COMMON::error ( "Unable to move ${vfstab_file}.tmp ${vfstab_file}");  
}

#------------------------------------------------------------------------------
#
# update /etc/vfstab to replace physical slice name by metadevice name
#
#------------------------------------------------------------------------------

sub update_vfstab {

  my $node_ip = COMMON::get_external_ip($NODE_ID);
  my $node_idx = COMMON::getnode($NODE_ID);
  
  my $local_vfstab = "$WORKING_DIR/vfstab";
  my $remote_vfstab = "/etc/vfstab";
  
  COMMON::print_action("Updating /etc/vfstab");
  #
  # retrieve /etc/vfstab file
  #
  COMMON::get_file($node_ip, "$remote_vfstab", "$local_vfstab");
  
  foreach my $i (@{$NODE_LIST[$node_idx]{slice}}) {

    if (($SLICE_LIST[$i]{metadevice} ne "") 
	&& ($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED)
	&& ($SLICE_LIST[$i]{mounting_point} ne $SLICE_TAG_REPLICA)) {
      COMMON::print_action("updating entry for $SLICE_LIST[$i]{name}");
      update_vfstab_entry($local_vfstab, 
			  $SLICE_LIST[$i]{name},
			  $SLICE_LIST[$i]{block},
			  $SLICE_LIST[$i]{raw});
    }
  }
  
  #
  # putback modified /etc/vfstab
  #
  COMMON::put_file($node_ip, "$local_vfstab", "$remote_vfstab");
}

#------------------------------------------------------------------------------
#
# mount newly created slices
#
#------------------------------------------------------------------------------

sub mount_slice {

  my $node_ip = COMMON::get_external_ip($NODE_ID);
  my $node_idx = COMMON::getnode($NODE_ID);
  
  COMMON::print_action("Mounting new slices") ;
  foreach my $i (@{$NODE_LIST[$node_idx]{slice}}) {

    if (($SLICE_LIST[$i]{metadevice} ne "")
	&& ($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED)
	&& ($SLICE_LIST[$i]{mounting_point} ne $SLICE_TAG_REPLICA)) {
      COMMON::print_subaction("mounting $SLICE_LIST[$i]{mounting_point} on $SLICE_LIST[$i]{block}");
      COMMON::remote_exec($node_ip, "$RMOUNT $SLICE_LIST[$i]{mounting_point}") ||
	COMMON::error ( "Unable to mount $SLICE_LIST[$i]{mounting_point}");
    }
  }
}

#------------------------------------------------------------------------------
#
# undo for recovery
#
#------------------------------------------------------------------------------

sub undo_svm {

  my $node_ip = COMMON::get_external_ip($NODE_ID);
  my $node_idx = COMMON::getnode($NODE_ID);

  # remove metadevice definition

  foreach my $i (@{$NODE_LIST[$node_idx]{slice}}) {
    if ($SLICE_LIST[$i]{metadevice} ne "") {

      COMMON::print_action("removing slice $SLICE_LIST[$i]{name} adaptation");

      # unmount partition if mounted
      # ignore error, it may be not mounted
      if (($SLICE_LIST[$i]{mounting_point} ne $SLICE_UNMOUNTED)
	  && ($SLICE_LIST[$i]{mounting_point} ne $SLICE_TAG_REPLICA)) {
COMMON::print_subaction("umount $SLICE_LIST[$i]{mounting_point}");
	COMMON::remote_exec_ignore($node_ip, "$RUMOUNT $SLICE_LIST[$i]{mounting_point}") ;
      }

      # remove the metadevice partition
      COMMON::print_subaction("remove metadevice $SLICE_LIST[$i]{metadevice}") ;
      COMMON::remote_exec_ignore($node_ip, "$RMETACLEAR $SLICE_LIST[$i]{metadevice}");
    }
  }

  # remove the meta database
  COMMON::remote_exec_ignore($node_ip, "$RMETADB -f -d $SLICE_REPLICA") ;
  
}

#------------------------------------------------------------------------------
#
# installation and configuration of SVM
#
#------------------------------------------------------------------------------

sub men_svm {

  ($NODE_ID) = @_;

  if (! $USE_SVM) {
    return;
  }
  
  COMMON::print_stage_node("DiskSuite configuration");
  configure_svm();
  update_vfstab();
  mount_slice();
}

#------------------------------------------------------------------------------
#
# recovery after failure
#
#------------------------------------------------------------------------------

sub recovery_men_svm {

  ($NODE_ID) = @_;

  if (! $USE_SVM) {
    return;
  }

  
  COMMON::print_recovery_node("Disksuite configuration clean-up");
  undo_svm();
}

{
}







