#!/usr/bin/perl
# -----------------------------------------------------------------------------
#   ident "@(#)reboot.pl 1.7 04/09/23 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package REBOOT;

use strict;
use FileHandle;
use variables;

#--------------------------------------------------------------
#
#  reboot a MEN: called by other functions
#
#--------------------------------------------------------------

sub generic_reboot {

  ($NODE_ID) = @_;

  if (($REBOOT ne "SOL") && ($REBOOT ne "FS")) {
    COMMON::error("Reboot: invalid type of reboot ($REBOOT)");
  }

  my $node_idx = COMMON::getnode($NODE_ID);
  my $node_ip = COMMON::get_external_ip($NODE_ID);
  my $node_name = COMMON::get_external_name($NODE_ID);

  COMMON::print_action("remotely reboot $node_name");

  # reboot the node
  my $result = qx{rsh -n $node_ip \"/usr/sbin/shutdown -y -g 0 -i 6\" 2>&1};

  # wait the reboot to be performed (if the ping comes too soon, the node is still there
  sleep 45;
  
  COMMON::print_action("waiting for network interface to be reachable");

  autoflush STDOUT 1;
  print "waiting.";
  for (;;) {
    $result = qx/ping $node_ip/;
    if ($result =~ /alive/) {
      last ;
    }
    sleep 10 ;
    print ".";
    
  }		    
  print "done.\n" ;
  autoflush STDOUT 0;
  
  my $process = "";
  if ($REBOOT eq "SOL") {
    COMMON::print_action("waiting for Solaris to be ready");
    $process = "snmpdx" ;
    
  } else {
    COMMON::print_action("waiting for Foundation Services to be ready");
    $process = "snmpdx" ;
  }

  # wait for a process to be there
  autoflush STDOUT 1;
  print "waiting.";
  for (;;) {
    $result = qx/rsh -n $node_ip \"pgrep $process\" 2>&1/;
    if ($result =~ /^[0-9]+$/) {
      last ;
    }
    print ".";
    sleep 10;
  }
  
  my $node_type = $NODE_LIST[$node_idx]{type};
  # check that CMM is correctly initialized
  if ($REBOOT eq "FS") {
    for (;;) {
      $result = qx{rsh -n $node_ip \"$CMMROLE -v\"};
      if ($node_type == $MEN_NODE) {
	if ($result =~ /current role [VICE_]*MASTER/) {
	  last ;
	}
      } else {
	if ($result =~ /current role IN_CLUSTER/) {
	  last ;
	}
      }
      print ".";
      sleep 10;
    }
  }
  print "done.\n";
  
  autoflush STDOUT 0;
}

#--------------------------------------------------------------
#
#  action: reboot a MEN
#
#--------------------------------------------------------------

sub reboot {

  ($NODE_ID) = @_;

  COMMON::print_stage_node("Reboot") ;

  generic_reboot($NODE_ID);
}

{
}
